/*
 * relay.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <vdr/plugin.h>
#include <vdr/shutdown.h>

static const char *VERSION        = "0.1.0";
static const char *DESCRIPTION    = "Power the output equipment on and off";

class cPluginRelay : public cPlugin {
private:
  int relayFD;
  ///< Open file descriptor of the relay device
  bool suspend;
  ///< Flag: suspend the output
public:
  cPluginRelay(void);
  virtual ~cPluginRelay();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual void MainThreadHook(void);
protected:
  inline void SetRelay(void);
  ///< Receive a Suspend event
  ///< @param Suspend true=output suspended, false=output running
  };

cPluginRelay::cPluginRelay(void) :
  relayFD (-1), suspend (false)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginRelay::~cPluginRelay()
{
  if (relayFD >= 0 && close(relayFD))
     perror ("[relay] close");
  relayFD = -1;
}

const char *cPluginRelay::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return "  -r DEV,   --relay=DEV    control the TxD line of DEV, e.g., /dev/ttyS0\n"
         "  -s[B],    --suspend[=B]  set the Suspend flag [to B]\n";
}

bool cPluginRelay::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  static const struct option long_options[] = {
       { "relay",    required_argument, NULL, 'r' },
       { "suspend",  optional_argument, NULL, 's' },
       { NULL }
     };

  int c;
  while ((c = getopt_long(argc, argv, "r:s::", long_options, NULL)) != -1) {
        switch (c) {
          case 'r': relayFD = open(optarg, O_RDWR | O_NOCTTY | O_NONBLOCK);
                    if (relayFD < 0) {
                       esyslog("[relay] open(\"%s\"): %m", optarg);
                       return false;
                       }
                    break;
          case 's': suspend = !optarg || atoi(optarg);
                    break;
          default:  return false;
          }
        }
  return true;
}

void cPluginRelay::SetRelay(void)
{
  isyslog("[relay] %s", suspend ? "suspend" : "resume");

  if (ioctl(relayFD, suspend ? TIOCCBRK : TIOCSBRK, 0))
     esyslog("[relay] ioctl(%s): %m", suspend ? "TIOCCBRK" : "TIOCSBRK");
}

bool cPluginRelay::Start(void)
{
  // Start any background activities the plugin shall perform.
  if (relayFD < 0) {
     esyslog("[relay] FATAL: missing or invalid -r or --relay argument!");
     return false;
     }
  SetRelay();
  return true;
}

void cPluginRelay::MainThreadHook(void)
{
  if (suspend == Shutdown.IsUserInactive())
     return;
  suspend = !suspend;
  SetRelay();
}

VDRPLUGINCREATOR(cPluginRelay); // Don't touch this!
